@@ -1,21 +1,34 @@
 from chamber.exceptions import PersistenceException
 
-from pymess.backend import BaseBackend, send_template as _send_template, send as _send
-from pymess.config import get_email_template_model, get_email_sender, settings
+from pymess.backend import BaseBackend, send_template as _send_template, BaseController
+from pymess.config import (
+    CONTROLLER_TYPES, get_email_template_model, is_turned_on_email_batch_sending, settings,
+)
 from pymess.models import EmailMessage
 
 
-class EmailBackend(BaseBackend):
-    """
-    Base class for E-mail backend containing implementation of e-mail service that
-    is used for sending messages.
-    """
+class EmailController(BaseController):
+    """Controller class for E-mail delegating message to correct E-mail backend"""
 
     model = EmailMessage
+    backend_type_name = CONTROLLER_TYPES.EMAIL
 
     class EmailSendingError(Exception):
         pass
 
+    def get_batch_size(self):
+        return settings.EMAIL_BATCH_SIZE
+
+    def get_batch_max_seconds_to_send(self):
+        return settings.EMAIL_BATCH_MAX_SECONDS_TO_SEND
+
+    def get_initial_email_state(self, recipient):
+        """
+        returns initial state for logged e-mail message.
+        :param recipient: e-mail address of the recipient
+        """
+        return self.model.STATE.WAITING
+
     def create_message(self, sender, sender_name, recipient, subject, content, related_objects, tag, template,
                        attachments, priority=settings.DEFAULT_MESSAGE_PRIORITY, **kwargs):
         """
@@ -30,6 +43,7 @@ class EmailBackend(BaseBackend):
         :param tag: string mark that will be saved with the message
         :param template: template object from which content, subject and sender of the message was created
         :param attachments: list of files that will be sent with the message as attachments
+        :param priority: priority of sending message 1 (highest) to 3 (lowest)
         :param kwargs: extra data that will be saved in JSON format in the extra_data model field
         """
         try:
@@ -45,7 +59,7 @@ class EmailBackend(BaseBackend):
                 state=self.get_initial_email_state(recipient),
                 priority=priority,
                 extra_data=kwargs,
-                **self._get_extra_message_kwargs()
+                **self.get_backend(recipient).get_extra_message_kwargs()
             )
             if attachments:
                 message.attachments.create_from_tripples(*attachments)
@@ -53,21 +67,6 @@ class EmailBackend(BaseBackend):
         except PersistenceException as ex:
             raise self.EmailSendingError(str(ex))
 
-    def is_turned_on_batch_sending(self):
-        return settings.EMAIL_BATCH_SENDING
-
-    def get_batch_max_number_of_send_attempts(self):
-        return settings.EMAIL_BATCH_MAX_NUMBER_OF_SEND_ATTEMPTS
-
-    def get_batch_max_seconds_to_send(self):
-        return settings.EMAIL_BATCH_MAX_SECONDS_TO_SEND
-
-    def get_batch_size(self):
-        return settings.EMAIL_BATCH_SIZE
-
-    def get_retry_sending(self):
-        return settings.EMAIL_RETRY_SENDING and self.is_turned_on_batch_sending()
-
     def send(self, sender, recipient, subject, content, sender_name=None, related_objects=None, tag=None,
              template=None, attachments=None, **kwargs):
         """
@@ -87,16 +86,26 @@ class EmailBackend(BaseBackend):
         message = self.create_message(
             sender, sender_name, recipient, subject, content, related_objects, tag, template, attachments, **kwargs
         )
+        backend = self.get_backend(message.recipient)
         if not self.is_turned_on_batch_sending():
-            self.publish_message(message)
+            backend.publish_message(message)
         return message
 
-    def get_initial_email_state(self, recipient):
-        """
-        returns initial state for logged e-mail message.
-        :param recipient: e-mail address of the recipient
-        """
-        return self.model.STATE.WAITING
+    def is_turned_on_batch_sending(self):
+        return is_turned_on_email_batch_sending()
+
+
+class EmailBackend(BaseBackend):
+    """
+    Base class for E-mail backend containing implementation of e-mail service that
+    is used for sending messages.
+    """
+
+    def get_batch_max_number_of_send_attempts(self):
+        return settings.EMAIL_BATCH_MAX_NUMBER_OF_SEND_ATTEMPTS
+
+    def get_retry_sending(self):
+        return settings.EMAIL_RETRY_SENDING and is_turned_on_email_batch_sending()
 
     def pull_message_info(self, message):
         """
@@ -145,7 +154,7 @@ def send(sender, recipient, subject, content, sender_name=None, related_objects=
     :param email_kwargs: extra data that will be saved in JSON format in the extra_data model field
     :return: True if e-mail was successfully sent or False if e-mail is in error state
     """
-    return get_email_sender().send(
+    return EmailController().send(
         sender=sender,
         recipient=recipient,
         subject=subject,
