@@ -7,8 +7,8 @@ from django.utils.timezone import now
 
 from chamber.utils.transaction import atomic_with_signals
 
-from pymess.config import get_email_sender, settings
-from pymess.models import EmailMessage
+from pymess.backend.emails import EmailController
+from pymess.config import settings
 
 
 logger = logging.getLogger(__name__)
@@ -24,9 +24,9 @@ class Command(BaseCommand):
         self.touched_message_pks = set()
         self.updated_messages = set()
 
-    def _get_messages_queryset_to_update(self):
+    def _get_messages_queryset_to_update(self, email_controller):
         delay = datetime.timedelta(seconds=settings.EMAIL_PULL_INFO_DELAY_SECONDS)
-        return EmailMessage.objects.filter(
+        return email_controller.model.objects.filter(
             # start_time = last_webhook_received_at + delay
             # info_changed_at < start_time
             # info_changed_at < last_webhook_received_at + delay
@@ -36,8 +36,8 @@ class Command(BaseCommand):
         ).order_by('-sent_at')
 
     @atomic_with_signals
-    def _pull_message_info(self, email_sender):
-        message = self._get_messages_queryset_to_update().exclude(
+    def _pull_message_info(self, email_controller):
+        message = self._get_messages_queryset_to_update(email_controller).exclude(
             pk__in=self.touched_message_pks
         ).select_for_update().first()
 
@@ -45,7 +45,7 @@ class Command(BaseCommand):
             return False
 
         self.touched_message_pks.add(message.pk)
-        email_sender.pull_message_info(message)
+        email_controller.get_backend(message.recipient).pull_message_info(message)
         self.updated_messages.add(message.pk)
         return True
 
@@ -56,11 +56,11 @@ class Command(BaseCommand):
             self.stdout.write('{}: {}'.format(title, len(message_pks)))
 
     def handle(self, *args, **options):
-        email_sender = get_email_sender()
+        email_controller = EmailController()
 
         for _ in range(settings.EMAIL_PULL_INFO_BATCH_SIZE):
             try:
-                if not self._pull_message_info(email_sender):
+                if not self._pull_message_info(email_controller):
                     break
             except Exception as ex:
                 # One message error should not stop the whole cycle
